/* USB console example

   This example code is in the Public Domain (or CC0 licensed, at your option.)

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include "esp_system.h"
#include "esp_log.h"
#include "esp_console.h"
#include "linenoise/linenoise.h"
#include "argtable3/argtable3.h"

#include "esp_vfs_cdcacm.h"


void register_cmds();

static void initialize_console(void)
{
    // Disable buffering on stdin
    setvbuf(stdin, NULL, _IONBF, 0);

    // Minicom, screen, idf_monitor send CR when ENTER key is pressed
    esp_vfs_dev_cdcacm_set_rx_line_endings(ESP_LINE_ENDINGS_CR);

    // Move the caret to the beginning of the next line on '\n'
    esp_vfs_dev_cdcacm_set_tx_line_endings(ESP_LINE_ENDINGS_CRLF);

    // Enable non-blocking mode on stdin and stdout
    fcntl(fileno(stdout), F_SETFL, 0);
    fcntl(fileno(stdin), F_SETFL, 0);

    // Initialize the console
    esp_console_config_t console_config = {
            .max_cmdline_args = 8,
            .max_cmdline_length = 256,
            .hint_color = atoi(LOG_COLOR_CYAN)
    };
    ESP_ERROR_CHECK( esp_console_init(&console_config) );

    // Configure linenoise line completion library 
    // Enable multiline editing. If not set, long commands will scroll within
    // single line.
    linenoiseSetMultiLine(1);

    // Tell linenoise where to get command completions and hints 
    linenoiseSetCompletionCallback(&esp_console_get_completion);
    linenoiseSetHintsCallback((linenoiseHintsCallback*) &esp_console_get_hint);

    // Set command history size 
    linenoiseHistorySetMaxLen(4);

    // Register commands 
    esp_console_register_help_command();

    register_cmds();
}

void pd_console_enter_loop()
{
    initialize_console();

    // Prompt to be printed before each line.
    // This can be customized, made dynamic, etc.
    const char* prompt = LOG_COLOR_I CONFIG_IDF_TARGET "> " LOG_RESET_COLOR;

    printf("=== Console Started ===\n" \
           "Type 'help' - get the list of commands.\n" \
           "UP/DOWN - navigate command history.\n" \
           "TAB - auto-complete.\n");

    // Figure out if the terminal supports escape sequences
    int probe_status = linenoiseProbe();
    if (probe_status) { 

        printf("Your terminal application does not support escape sequences.\n"
               "Line editing and history features are disabled.\n");

        linenoiseSetDumbMode(1);

        // Since the terminal doesn't support escape sequences,
        // don't use color codes in the prompt.
        prompt = CONFIG_IDF_TARGET "> ";
    }

    while(true) {

        // Get a line using linenoise.
        // The line is returned when ENTER is pressed.
        char* line = linenoise(prompt);
        if (line == NULL) { // Ignore empty lines
            continue;
        }

        // Add the command to the history 
        linenoiseHistoryAdd(line);

        // Try to run the command 
        int ret;
        esp_err_t err = esp_console_run(line, &ret);
        if (err == ESP_ERR_NOT_FOUND) {

            printf("Unrecognized command\n");

        } else if (err == ESP_ERR_INVALID_ARG) {

            // command was empty

        } else if (err == ESP_OK && ret != ESP_OK) {

            printf("Command returned non-zero error code: 0x%x (%s)\n",
                ret, esp_err_to_name(ret));

        } else if (err != ESP_OK) {

            printf("Internal error: %s\n", esp_err_to_name(err));
        }

        // linenoise allocates line buffer on the 
        // heap, so need to free it
        linenoiseFree(line);
    }
}

//
// Commands
//

static struct {
    struct arg_str *ssid;
    struct arg_str *pass;
    struct arg_end *end;
} factory_wifi;

static int cmd_factory_wifi(int argc, char **argv)
{
    return 0;
}

void register_factory_wifi(){

    factory_wifi.ssid = arg_str1(NULL, NULL, "<ssid>", "ssid value");
    factory_wifi.pass = arg_str1(NULL, NULL, "<pass>", "password value");
    factory_wifi.end = arg_end(2);

    const esp_console_cmd_t set_cmd = {
        .command = "factory-wifi",
        .help = "Set the ssid & password to use in Factory Mode (AP Connect).\n",
        .hint = NULL,
        .func = &cmd_factory_wifi,
        .argtable = &factory_wifi
    };

    ESP_ERROR_CHECK(esp_console_cmd_register(&set_cmd));
}

void register_cmds(){

    register_factory_wifi();
}


